package biteme;

/**
 * Enumeration representing the five possible directions. Includes some useful functions.
 * @author NFW Thissen
 * @version 0.0.1
 */
public enum Direction {
    /** Represents no direction. */
    NONE(-1),
    /** Represents moving up. */
    UP(0),
    /** Represents moving down. */
    DOWN(1),
    /** Represents moving left. */
    LEFT(2),
    /** Represents moving right. */
    RIGHT(3);

    /** Enumeration value. */
    private int _value = 0;

    /**
     * Constructor.
     * @param value The enumeration value.
     */
    Direction(int value) {
        _value = value;
    }

    /**
     * Determines the Direction opposite to the specified Direction.
     * @param d The Direction to determine the opposite of.
     * @return The opposite Direction to the specified Direction.
     */
    public static Direction getOpposite(Direction d) {
        if (d == UP) {
            return DOWN;
        } else if (d == RIGHT) {
            return LEFT;
        } else if (d == DOWN) {
            return UP;
        } else if (d == LEFT) {
            return RIGHT;
        } else {
            return NONE;
        }
    }

    /**
     * Determines a Direction from two different tiles. Tiles must be in the same row or column.
     * @param current The Tile to start calculation.
     * @param other The Tile to stop calculation.
     * @return The Direction between Tile current and Tile other, if they are in the same row or column and not null.
     * Otherwise, Direction.NONE.
     */
    public static Direction fromTiles(Tile current, Tile other) {
        if (current == other || current == null || other == null) {
            return NONE;
        }

        int x1 = current.getLocationX();
        int x2 = other.getLocationX();
        int y1 = current.getLocationY();
        int y2 = other.getLocationY();

        if (x1 == x2 && y1 < y2) {
            return DOWN;
        } else if (x1 == x2 && y1 > y2) {
            return UP;
        } else if (x1 < x2 && y1 == y2) {
            return RIGHT;
        } else if (x1 > x2 && y1 == y2) {
            return LEFT;
        } else {
            return NONE;
        }
    }
}
