/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package biteme;


/**
 * Tile representing an empty tile, which can contain a coin or box.
 * @author N.F.W. Thissen, A.M. Zantema
 * @version 0.0.1
 */
public class EmptyTile extends Tile {

    /**
     *
     * @param m
     */
    public EmptyTile(Maze m, int x, int y) {
        super(m, x, y);
    }

    /**
     * This method checks whether this tile contains a ghost or not.
     * @return true if there is a ghost, false if there is no ghost.
     */
    public boolean containsGhost() {
        return (this.containsSmartGhost() || this.containsStupidGhost());
//        for (SmartGhost g : maze.getSmartGhosts()) {
//            if (g.getTile().equals(this)) {
//                return true;
//            }
//        }
//        for (StupidGhost g : maze.getStupidGhosts()) {
//            if (g.getTile().equals(this)) {
//                return true;
//            }
//        }
//
//        return false;
    }

    /**
     * This function checks whether this tile contains a smart ghost or not.
     * @return true if there is a smart ghost, else false.
     */
    public boolean containsSmartGhost() {
        for (SmartGhost g : maze.getSmartGhosts()) {
            if (g.getTile().equals(this)) {
                return true;
            }
        }
        return false;
    }

    /**
     * This function checks whether this tile contains a stupid ghost or not.
     * @return true if there is a stupid ghost, else false.
     */
    public boolean containsStupidGhost() {
        for (StupidGhost g : maze.getStupidGhosts()) {
            if (g.getTile().equals(this)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Checks whether an entity can leave a tile in a certain direction.
     * @param d The direction in which the entity wants to leave the tile.
     * @param e The entity that wants to leave the tile.
     * @return true. An entity can always leave an empty tile.
     */
    public boolean canLeave(Direction d, Entity e) {
        return true;
    }

    /**
     * Checks whether an entity can enter the tile.
     * @param d The Direction to check for.
     * @param e The Entity to check for.
     * @return If the Tile doesn't contain a box, the Entity can enter.
     */
    public boolean canEnter(Direction d, Entity e) {
        return !this.getStatus().containsBox();
    }

    /**
     * Retrieves the first empty tile to which a box can be moved.
     * If this tile contains no coin and no ghost but has a box, then the
     * getNextBoxTile function is called for the next tile and its return value
     * is returned.
     * If this tile contains no coin, no ghost and no box, this tile itslf is the
     * first empty tile to which a box can be moved (this tile will be returned).
     * If this contains a coin or a ghost, a box cannot be moved here and the value
     * null is returned.
     * @param d The direction in which the box will be moved (if possible).
     * @return The first empty tile to which a box can be moved, or null if no such tile exists.
     */
    public Tile getNextBoxTile(Direction d) {
        //######################################################################
        //####### IMPLEMENTATION DIFFERS FROM DESCRIPTION BELOW ################
        //######################################################################
        // retrieve (recursively) the first empty tile to which a box can be moved.
        // If no empty tile exists, return null, meaning the box cannot move.
        
        // If this tile contains a coin, return null (box cannot move on coin)
        
        // If the NEXT tile contains a box, 
        // recursively call the getNextBoxTile function on THAT tile (and return its return value)

        // If the NEXT tile does not contain a box, return the current tile.
        //######################################################################
        //####### IMPLEMENTATION DIFFERS FROM DESCRIPTION ABOVE ################
        //######################################################################

        Tile nextTile = maze.getNextTile(this, d);

        if ((!this.getStatus().containsCoin()) && (!this.containsGhost()) && (d != Direction.NONE)) {
            if (this.getStatus().containsBox()) {
                return nextTile.getNextBoxTile(d);
            } else {
                return this;
            }
        } else {
            return null;
        }
    }

    /**
     * Draws this tile (or the box, coin or ghost at this tile).
     */
    public void draw() {
        if (this.getStatus().containsBox()) {
            maze.drawAtXY("box", getLocationX(), getLocationY());
        } else {
            if (this.getStatus().containsCoin()) {
                if (!this.containsGhost()) {
                    maze.drawAtXY("coin", getLocationX(), getLocationY());
                } else {
                    if (this.containsSmartGhost()) {
                        maze.drawAtXY("SmartGhostC", getLocationX(), getLocationY());
                    } else {
                        maze.drawAtXY("StupidGhostC", getLocationX(), getLocationY());
                    }
                }
            } else {
                if (!this.containsGhost()) {
                    maze.drawAtXY("black", getLocationX(), getLocationY());
                } else {
                    if (this.containsSmartGhost()) {
                        maze.drawAtXY("SmartGhost", getLocationX(), getLocationY());
                    } else {
                        maze.drawAtXY("StupidGhost", getLocationX(), getLocationY());
                    }
                }
            }
        }
    }
}
