
package biteme;


/**
 * Base class for all movable entities such as Pacman and Ghosts.
 * @author NFW Thissen
 * @version 0.0.1
 */
public abstract class Entity {

    /** The Tile this Entity is currently on. */
    protected Tile tile;
    
    /** The GameGUI instance. */
    protected Maze maze;

    /** The current direction of this entity. */
    protected Direction direction;

    /**
     * Constructor. 
     * @param m The Maze instance.
     */
    public Entity(Maze m) {
        this.maze = m;

        direction = Direction.NONE;
    }

    /**
     * Gets the Tile this Entity is currently on.
     * @return The Tile this Entity is currently on.
     */
    public Tile getTile() {
        return tile;
    }

     /**
     * Gets the Tile this Entity is currently on.
      * @param t
     */
    public void setTile(Tile t) {
        tile = t;
    }

    /**
     * Move this entity in the specified direction.
     * @param direction The direction to move in.
     */
    public void move(Direction direction) {
        Tile oldTile = this.getTile();
        Tile newTile = maze.getNextTile(oldTile, direction);
        if (newTile.canEnter(direction, this) && oldTile.canLeave(direction, this)) {
            this.setTile(newTile);

            // Draw both tiles + entity
            oldTile.draw();
            newTile.draw();
            this.draw();
        }
    }

    /**
     * Draws the entity on its Tile.
     */
    public abstract void draw();

}
