package biteme;

import java.util.LinkedList;

/**
 * Base class for both ghost entities.
 * @author NFW Thissen
 * @version 0.0.1
 */
public abstract class GhostEntity extends Entity {

    /**
     * Constructor.
     * @param m The GameGUI instance.
     */
    public GhostEntity(Maze m) {
        super(m);
    }

    /**
     * Checks every Direction and adds it to a list if this Ghost can enter the next Tile in that Direction.
     * @return A list of all possible Directions.
     */
    protected LinkedList<Direction> getPossibleDirections() {
        LinkedList<Direction> possibleDirs = new LinkedList<Direction>();
        Tile nextTile;

        // Loop through all directions
        for (Direction d : Direction.values()) {
            if (d != Direction.NONE) {
                // If the ghost can enter the next tile in this direction, add it to the list.
                nextTile = maze.getNextTile(this.getTile(), d);
                if (nextTile.canEnter(d, this) && this.getTile().canLeave(d, this)) { //&& d != Direction.getOpposite(this.direction)) {
                    possibleDirs.add(d);
                }
            }
        }
        if(possibleDirs.isEmpty())
            possibleDirs.add(Direction.getOpposite(this.direction));

        return possibleDirs;
    }

    /**
     * Chooses a Direction to move in.
     * @return A Direction to move in.
     */
    public abstract Direction chooseDirection();
}
