/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package biteme;

import java.util.Arrays;
import java.util.Enumeration;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;

/**
 * Provides a way of displaying a List of Strings in a scrollable container.
 * @author N.F.W. Thissen
 */
public class ListBox extends JList {

    /** Enumeration providing a choice between descending and ascending sort order. */
    public enum SortOrder {
        /** Represent sorting in descending order. */
        DESCENDING (0),
        /** Represents sorting in ascending order. */
        ASCENDING (1);

        /** Private enumeration value. */
        private int value = 0;

        /** Constructor for the enumeration. */
        SortOrder(int val) {
            value = val;
        }
    }

    /** The DefaultListModel object used internally. */
    private DefaultListModel listModel;

    /**
     * Constructor. Creates a new DefaultListModel and sets common properties.
     */
    public ListBox() {
        listModel = new DefaultListModel();

        this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.setLayoutOrientation(JList.VERTICAL);

        this.setModel(listModel);
    }

    /**
     * Adds a String to the displayed List.
     * @param name The value of the String to add.
     */
    public void addLevel(String name) {
            listModel.addElement(name);
            this.setModel(listModel);
    }

    /**
     * Removes a String from the displayed List, if present.
     * @param name The display value of the String to remove.
     */
    public void removeLevel(String name) {
            listModel.removeElement(name);
            this.setModel(listModel);
    }

    /**
     * Removes a String from the displayed List at the specified index. Does not throw IndexOutOfBounds exception;
     * indices out of bounds are ignored.
     * @param index The index of the String to remove.
     */
    public void removeLevelAt(int index) {
        if (index >= 0 && index < listModel.size()) {
            listModel.removeElementAt(index);
            this.setModel(listModel);
        }
    }

    /**
     * Removes all Strings in the displayed List.
     */
    public void clear() {
        listModel.clear();
        this.setModel(listModel);
    }

    /**
     * Gets a String array of the displayed List.
     * @return A String array of the displayed List.
     */
    public String[] items() {
        String[] s = new String[listModel.size()];
        Enumeration<?> elements = listModel.elements();
        int i = 0;
        while (elements.hasMoreElements()) {
            s[i] = elements.nextElement().toString();
            i++;
        }
        return s;
    }

    /**
     * Checks if the specified display value is present in the displayed List.
     * @param name The display value to check for.
     * @return True if the specified display value is present in the displayed List.
     */
    public boolean containsLevel(String name) {
        return listModel.contains(name);
    }

    /**
     * Gets the zero-based index of a specified display value.
     * @param name The String to search for.
     * @return The zero-based index of the specified String if present,
     * or -1 if the String is not in the displayed List.
     */
    public int indexOf(String name) {
        return listModel.indexOf(name);
    }

    /**
     * Gets the level at a specified index.
     * @param index The zero-based index.
     * @return The String in the displayed List at the specified index.
     */
    public String getLevelAtIndex(int index) {
        return listModel.get(index).toString();
    }

    /**
     * Sorts the Strings in the displayed List in the specified order.
     * @param so The sorting order (SortOrder.ASCENDING or SortOrder.DESCENDING) to use.
     */
    public void sort(SortOrder so) {
        String[] items = this.items();
        listModel.clear();
        Arrays.sort(items);

        if (so == SortOrder.DESCENDING) {
            for (int i = 0; i < items.length; i++) {
                listModel.addElement(items[i]);
            }
        } else {
            for (int i = items.length - 1; i >= 0; i--) {
                listModel.addElement(items[i]);
            }
        }

        this.setModel(listModel);
    }
}
