/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package biteme;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Thread class that loads a maze asynchronously.
 * @author N.F.W. Thissen
 */
public class LoadMazeThread extends Thread {

    /** The Maze object to load the level in. */
    private Maze _maze = null;

    /** The path of the maze file to load. */
    private String _filePath = null;

    /** A list of objects to notify when the maze has finished loading. */
    private List<ILoadMazeThreadListener> listeners = new ArrayList<ILoadMazeThreadListener>();

    /**
     * Constructor. Creates a new instance of this thread class.
     * @param maze The Maze object to load the level in.
     * @param filePath The path of the maze file to load.
     */
    public LoadMazeThread(Maze maze, String filePath) {
        _maze = maze;
        _filePath = filePath;
    }

    /** Sets the filepath of the level to load. */
    public void setFilePath(String filePath) {
        _filePath = filePath;
    }

    /**
     * Adds an object to notify when loading has finished.
     * @param listener An object implementing ILoadMazeThreadListener to notify when loading has finished.
     */
    public void addLoadMazeListener(ILoadMazeThreadListener listener) {
        listeners.add(listener);
    }

    /**
     * Removes an object to notify when loading has finished.
     * @param listener An object implementing ILoadMazeThreadListener to stop notifying when loading has finished.
     */
    public void removeLoadMazeListener(ILoadMazeThreadListener listener) {
        listeners.remove(listener);
    }

    /** Runs this thread. Do NOT call this method directly. Call start() instead. */
    @Override public void run() {
        if (_maze != null && _filePath != null) {
            _maze.clear();
            MazeIO.loadMaze(_maze, _filePath);
            _maze.initialDraw();
        }
        this.fireThreadFinished();
    }
    
    /** Notifies all listeners that loading has finished. */
    private void fireThreadFinished() {
        for (Iterator<ILoadMazeThreadListener> it = listeners.iterator(); it.hasNext();) {
            it.next().loadFinished();
        }
    }
}


