/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package biteme;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.net.URL;
import java.util.LinkedList;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * Represents a collection of Tiles forming a level.
 * @author NFW Thissen
 * @version 0.0.1
 */
public class Maze extends JPanel {

    /** The dimension of a tile, in pixels. */
    private int tileSize = 30;

    /** All tiles in this Maze. */
    private LinkedList<Tile> tiles;

    /** The PacmanEntity in this Maze. */
    private PacmanEntity pacman;

    /** All StupidGhost entities in this Maze. */
    private LinkedList<StupidGhost> stupidGhosts;

    /** All SmartGhost entities in this Maze. */
    private LinkedList<SmartGhost> smartGhosts;

    /** The labels representing tiles. */
    public JLabel[][] grid = new JLabel[30][20];

    /** The parent JFrame of this Maze. */
    private JFrame parent = null;

    /** The background image of this Maze. */
    public Image backgroundImage = null;

    /**
     * Constructor. Initializes collections, loads background image and creates an empty maze.
     */
    public Maze() {
        tiles = new LinkedList<Tile>();
        smartGhosts = new LinkedList<SmartGhost>();
        stupidGhosts = new LinkedList<StupidGhost>();

        // Load background image (walls)
        backgroundImage = Toolkit.getDefaultToolkit().getImage(MainMenu.class.getResource("maze_background.gif"));

        for (int x = 0; x < 30; x++) {
            for (int y = 0; y < 20; y++) {
                tiles.add(new EmptyTile(this, x, y));
            }
        }
    }

    /**
     * Sets the parent.
     * @param pParent The new parent.
     */
    void setParent(JFrame pParent) {
        parent = pParent;
    }

    /**
     * Gets the reference to Pacman.
     * @return The reference to Pacman.
     */
    public PacmanEntity getPacman() {
        return pacman;
    }

    /**
     * Sets the reference to Pacman. Only to be used when loading a Maze.
     * @param p A new PacmanEntity.
     */
    public void setPacman(PacmanEntity p) {
        pacman = p;
    }

    /**
     * Gets all StupidGhosts.
     * @return A LinkedList of all StupidGhosts.
     */
    public LinkedList<StupidGhost> getStupidGhosts() {
        return stupidGhosts;
    }

    /**
     * Gets all SmartGhosts.
     * @return A LinkedList of all SmartGhosts.
     */
    public LinkedList<SmartGhost> getSmartGhosts() {
        return smartGhosts;
    }

    /**
     * Gets the size, in pixels, of a Tile (default: 30), to be used to display a smaller 'preview' Maze.
     * @return The size, in pixels, of a Tile.
     */
    public int getTileSize() {
        return tileSize;
    }

    /**
     * Sets the size, in pixels, of a Tile (default: 30), to be used to display a smaller 'preview' Maze.
     * @param size The size, in pixels, of a Tile.
     */
    public void setTileSize(int size) {
        tileSize = size;
    }

    /**
     * Gets all Tiles in the Maze. Do not rely on their order.
     * @return A LinkedList of all Tiles.
     */
    public LinkedList<Tile> getTiles() {
        return tiles;
    }

    /**
     * Gets the number of coins still present in the Maze.
     * @return The number of coins in the Maze.
     */
    public int getCoinCount() {
        int i = 0;
        for (Tile t : tiles) {
            if (t.getStatus().containsCoin()) {
                i++;
            }
        }
        return i;
    }

    /**
     * Adds a new SmartGhost entity to this Maze at the specified coordinates.
     * @param x The x-coordinate.
     * @param y The y-coordinate.
     */
    public void addSmartGhost(int x, int y) {
        SmartGhost g = new SmartGhost(this);
        this.addGhost(g, x, y);
    }

    /**
     * Adds a new StupidGhost entity to this Maze at the specified coordinates.
     * @param x The x-coordinate.
     * @param y The y-coordinate.
     */
    public void addStupidGhost(int x, int y) {
        StupidGhost g = new StupidGhost(this);
        this.addGhost(g, x, y);
    }

    /**
     * Adds a GhostEntity (doesn't matter which type) to this Maze. Make sure you manually set its Tile!
     * @param g The GhostEntity to add.
     */
    public void addGhost(GhostEntity g) {
        if (g instanceof SmartGhost) {
            smartGhosts.add((SmartGhost) g);
        } else {
            stupidGhosts.add((StupidGhost) g);
        }
    }

    /**
     * Adds a SmartGhost to the Maze at the Tile at coordinates x, y.
     * @param smg The SmartGhost to add to the Maze.
     * @param x The x-coordinate of the target Tile.
     * @param y The y-coordinate of the target Tile.
     */
    public void addGhost(GhostEntity g, int x, int y) {
        Tile t = this.getTile(x, y);
        if (t != null) {
            g.setTile(t);
            this.addGhost(g);
        }
    }

    /**
     * Removes a specified GhostEntity (of any type) from the Maze, if it exists.
     * @param g The GhostEntity to remove.
     */
    public void removeGhost(GhostEntity g) {
        if (g instanceof SmartGhost) {
            SmartGhost smg = (SmartGhost) g;
            if (smartGhosts.contains(smg)) {
                smartGhosts.remove(smg);
            }
        } else {
            StupidGhost smg = (StupidGhost) g;
            if (stupidGhosts.contains(smg)) {
                stupidGhosts.remove(smg);
            }
        }
    }

    /**
     * Removes ALL GhostEntities, if any, from the specified Tile.
     * @param t The Tile to remove all GhostEntities from.
     */
    public void removeGhostsFromTile(Tile t) {
        LinkedList<GhostEntity> ghostsToRemove = new LinkedList<GhostEntity>();

        for (SmartGhost g : smartGhosts) {
            if (g.getTile() == t) {
                ghostsToRemove.add(g);
            }
        }
        for (StupidGhost g : stupidGhosts) {
            if (g.getTile() == t) {
                ghostsToRemove.add(g);
            }
        }
        for (GhostEntity g : ghostsToRemove) {
            removeGhost(g);
        }
    }

    /**
     * Clears the Maze. Removes every Entity and fills the Maze with EmptyTiles.
     */
    public void clear() {
        pacman = null;
        stupidGhosts = new LinkedList<StupidGhost>();
        smartGhosts = new LinkedList<SmartGhost>();

        int x, y;
        for (Tile t : tiles) {
            x = t.getLocationX();
            y = t.getLocationY();
            this.setTile(new EmptyTile(this, x, y), x, y);
        }
    }

    /**
     * Gets the Tile at the specified coordinates.
     * @param x The x-coordinate of the Tile, between 0 and 29.
     * @param y The y-coordinate of the Tile, between 0 and 19.
     * @return The Tile at coordinates (x, y).
     */
    public Tile getTile(int x, int y) {
        return tiles.get(xyToIndex(x, y));
    }

    /**
     * Gets the Tile at the specified absolute location (not coordinate), probably for a mouse click.
     * @param absX The absolute x-coordinate, between 0 and 29*tileSize.
     * @param absY The absolute y-coordinate, between 0 and 19*tileSize.
     * @return The Tile at the specified mouse location.
     */
    public Tile getTileFromMouseClick(int absX, int absY) {
        int x = (int) Math.floor(absX / 30);
        int y = (int) Math.floor(absY / 30);
        return this.getTile(x, y);
    }

    /**
     * Sets the Tile at the specified coordinates.
     * @param t The new Tile.
     * @param x The x-coordinate.
     * @param y The y-coordinate.
     */
    public void setTile(Tile t, int x, int y) {
        int index = this.xyToIndex(x, y);
        tiles.set(index, t);
    }

    /**
     * Gets the next tile in the specified Direction.
     * @param currentTile The Tile to start looking.
     * @param d The Direction in which to look.
     * @return The next Tile in the specified Direction if within bounds. A new WallTile otherwise.
     */
    public Tile getNextTile(Tile currentTile, Direction d) {
        int x = currentTile.getLocation().x;
        int y = currentTile.getLocation().y;

        switch (d) {
            case UP:
                y -= 1;
                break;
            case DOWN:
                y += 1;
                break;
            case LEFT:
                x -= 1;
                break;
            case RIGHT:
                x += 1;
                break;
            case NONE:
                break;
        }
        if ((x >= 0 && x <= 29) && (y >= 0 && y <= 19)) {
            return this.getTile(x, y);
        } else {
            // out of bounds
            return new WallTile(this, 0, 0);
        }

    }

    /** Executes Dijkstra's shortest path algorithm, telling each Tile the shortest path to Pacman. */
    public void executeDijkstra() {
        LinkedList<Tile> Q = tiles;

        // initialiseer elke Tile met dist = infinity en prev = null
        for (Tile t : Q) {
            t.dist = Integer.MAX_VALUE;
            t.setNextTileToPacman(null);
            t.isVisited = false;
        }

        // dist pacman = 0
        int pacmanIndex = tiles.indexOf(pacman.getTile());
        Q.get(pacmanIndex).dist = 0;

        boolean loop = true;
        while (loop) {
            Tile u = null;
            
            int smallest = Integer.MAX_VALUE;
            for (Tile t : Q) {
                if (!t.isVisited && t.dist <= smallest) {
                    smallest = t.dist;
                    u = t;
                }
            }
            
            if (u == null || u.dist == Integer.MAX_VALUE) {
                // stop
                loop = false;
            } else {
                u.isVisited = true;
                
                Tile nt = null;
                int alt;
                SmartGhost g = new SmartGhost(this);
                for (Direction d : Direction.values()) {
                    if (d != Direction.NONE) {
                        g.direction = d;
                        nt = this.getNextTile(u, d);

                        if (nt == null) System.out.println("nt = null");

                        if (nt.canEnter(Direction.getOpposite(d), g) && u.canLeave(Direction.getOpposite(d), g)) {
                            alt = u.dist + 1;
                            if (alt < nt.dist) {
                                nt.dist = alt;
                                nt.setNextTileToPacman(u);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Converts an (x, y) coordinate into the equivalent 1-dimensional array index.
     * @param x The x-coordinate.
     * @param y The y-coordinate.
     * @return The index value for the Tile with coordinates (x, y).
     */
    private int xyToIndex(int x, int y) {
        return x + 30 * y;
    }

    /**
     * Converts a 1-dimensional array index to the equivalent (x, y) coordinates.
     * @param index The index.
     * @return The (x, y) coordinates corresponding to the specified index.
     */
    private Point indexToXY(int index) {
        int x, y;
        x = index % 30;
        y = (int) Math.floor(index / 30);

        return new Point(x, y);
    }


    /**
     * Draws the specified icon on to the Tile with coordinates (x, y).
     * @param file The name of the image to draw, exluding its file extension.
     * @param x The x-coordinate of the Tile to draw on.
     * @param y The y-coordinate of the Tile to draw on.
     */
    public void drawAtXY(String file, int x, int y) {

        ImageIcon symbol = new ImageIcon();
        // New gridbag properties
        GridBagConstraints c = new GridBagConstraints();
        // set contraint to horizontal aligment
        c.fill = GridBagConstraints.HORIZONTAL;


        Image img = null;
        try {
            URL r = MainMenu.class.getResource(file + ".gif");
            img = Toolkit.getDefaultToolkit().getImage(r);
        }
        catch (Exception ex) {
            JOptionPane.showConfirmDialog((Component)
                null, ex.toString(), "alert", JOptionPane.DEFAULT_OPTION);
        }

        if (img == null) return;
        
        if (tileSize == 30) {
            // geen resize voor standaard grootte (sneller)            
            symbol = new ImageIcon(img);
        } else {
            // resize voor preview
            Image sizedImg = img.getScaledInstance(tileSize, tileSize, java.awt.Image.SCALE_SMOOTH);
            symbol = new ImageIcon(sizedImg);
        }


        // check of the component al bestaat, zoja verwijder de component
        if (this.isAncestorOf(grid[x][y])) {
            this.remove(grid[x][y]);
        }

        // create a new label
        JLabel j = new JLabel(symbol);
        j.setPreferredSize(new Dimension(tileSize, tileSize));
        grid[x][y] = j;
        grid[x][y].addMouseListener(new java.awt.event.MouseAdapter() {

            public void mousePressed(java.awt.event.MouseEvent evt) {
                //System.out.println(evt.getButton());
                if (parent != null && (parent instanceof EditorGUI)) {
                    // get x and y coordinates of the label
                    int x = (((JLabel) evt.getSource()).getX() / tileSize) - 1;
                    int y = (((JLabel) evt.getSource()).getY() / tileSize) - 1;
                    // tell the editor which tile to replace
                    if(evt.getButton() == 1) {
                        ((EditorGUI) parent).changeTileAt(x, y);
                    } else if(evt.getButton() == 3) {
                        ((EditorGUI) parent).removeTileAt(x, y);
                    }
                }
            }
        });

        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = x;// position x
        c.gridy = y;// position y
        this.add(grid[x][y], c);

        this.validate();
    }

    /**
     * Paints the background image on to the Maze.
     * @param g The Graphics object used to paint.
     */
    @Override public void paintComponent(Graphics g) {
        super.paintComponent(g);
        // only draw if the tileSize == 30, otherwise the preview size isn't correct
        if (backgroundImage != null && tileSize == 30) {
            g.drawImage(backgroundImage, 0, 0, null);
        }
    }

    /** Draws the Maze for the first time, forcing every Tile to redraw. */
    public void initialDraw() {
        for (Tile t : tiles) {
            t.draw();
        }
        for (SmartGhost g : smartGhosts) {
            g.draw();
        }
        for (StupidGhost g : stupidGhosts) {
            g.draw();
        }
        if (pacman != null) {
            pacman.draw();
        }
    }
}
