/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package biteme;

import java.awt.Point;
import java.io.*;
import java.util.*;

/**
 * Hosts static Save and Load methods to save and load maze files.
 * @author N.F.W. Thissen
 *         J.H. van der Aa
 */
public class MazeIO {

    /**
     * Saves a Maze to a file.
     * @param maze The Maze to save.
     * @param all the scorelist of all time scores for this maze
     * @param week the scorelist of the current weeks scores for this maze
     * @param filePath The path to store the file.
     * @param day the scorelist of today's scores for this maze
     */
    public static void saveMaze(Maze maze, ScoreList all, ScoreList week, ScoreList day, String filePath) {
        // voor elke tile, sla de status op
        int statusValue;
        String mazeCode = "";
        String pacman = "";
        String stghost = "";
        String smartghost = "";


        for (Tile t : maze.getTiles()) {
            statusValue = t.getStatus().getValue();
            mazeCode = mazeCode + " " + statusValue;
        }

        // sla positie van pacman op
        if (maze.getPacman() != null) {
            Point pacmanLocation = maze.getPacman().getTile().getLocation();
            pacman = pacmanLocation.x + " " + pacmanLocation.y + " ";
        }

        // sla positie van ghosts op
        Point stupidGhostLocation;
        for (StupidGhost stg : maze.getStupidGhosts()) {
            stupidGhostLocation = stg.getTile().getLocation();
            stghost = stghost + stupidGhostLocation.x + " " + stupidGhostLocation.y + " ";
        }

        Point smartGhostLocation;
        for (SmartGhost smg : maze.getSmartGhosts()) {
            smartGhostLocation = smg.getTile().getLocation();
            smartghost = smartghost + smartGhostLocation.x + " " + smartGhostLocation.y + " ";
        }




        String fileNameTarget = filePath;
        File target = new File(fileNameTarget);
        try {
            PrintWriter pw = new PrintWriter(target);
            pw.println(mazeCode);
            pw.println(pacman);
            pw.println("stupidghosts");
            pw.println(stghost);
            pw.println("smartghosts");
            pw.println(smartghost);

            //save scores
            pw.println("all");
            for (int i = 1; i < 11; i++) {
                int score = all.getScore(i).getScore();
                String date = all.getScore(i).getDateString();
                String name = all.getScore(i).getName();

                pw.println(score + " " + date + " " + name);
            }

            pw.println("week");
            for (int i = 1; i < 11; i++) {
                int score = week.getScore(i).getScore();
                String date = week.getScore(i).getDateString();
                String name = week.getScore(i).getName();

                pw.println(score + " " + date + " " + name);
            }

            pw.println("day");
            for (int i = 1; i < 11; i++) {
                int score = day.getScore(i).getScore();
                String date = day.getScore(i).getDateString();
                String name = day.getScore(i).getName();

                pw.println(score + " " + date + " " + name);
            }


            // Flush and close
            pw.flush();
            pw.close();

        } catch (FileNotFoundException e) {
            Popup.ok("Error: File not found");
        }
    }

    /**
     * Changes an unplayable file to a playable file
     * @param filepath the path to the original file
     * @return the path to the new file
     */
    public static String makePlayable(String filepath) {
        String newname = getFileName(filepath) + ".ply";
        String returnname = Path.getMazesPath() + newname;        

        copyFile(filepath, returnname);
        File oldFile = new File(filepath);
        oldFile.delete();
        
        return returnname;

    }

       /**
     * Changes a playable file to an unplayable file
        * @param filepath the path to the original file
        * @return the path to the new file
     */
    public static String makeUnPlayable(String filepath)  {
        String newname = getFileName(filepath) + ".unp";
        String returnname = Path.getMazesPath() + newname;
        copyFile(filepath, returnname);
        File oldFile = new File(filepath);
        oldFile.delete();

        return returnname;
    }

     /**
     * Renames a file
     * @param oldpath the path to the old file
      * @param newpath the path to the new file
      * @return the path of the new file
     */
    public static String renameLevel(String oldpath, String newpath){
        copyFile(oldpath, newpath);
        File oldFile = new File(oldpath);
        oldFile.delete();

        return newpath;
    }


    /**
     * 
     * @param filepath The path to extract the name from.
     * @return the filename without the extension and path
     */
    public static String getFileName(String filepath) {
        int dotPos = filepath.lastIndexOf(".");
        int slashPos = filepath.lastIndexOf("\\");
        return filepath.substring(slashPos + 1, dotPos);
    }

    /**
     * Loads a Maze from a file. Pass the Maze you want to load to.
     * @param maze The Maze to load to.
     * @param filePath The path to load the Maze from.
     */
    public static void loadMaze(Maze maze, String filePath) {
        int[] statusValues = new int[600];
        int x = 0;
        int y = 0;
        Status s;
        Tile t;
        Scanner scanner;
        File mazefile = new File(filePath);
        //scans the maze
        try {
            scanner = new Scanner(mazefile);

            for (int i = 0; i < 600; i++) {
                statusValues[i] = scanner.nextInt();
            }

            //maze.clear();

            //assigns statusvalues to tiles
            for (int statusValue : statusValues) {
                s = new Status();
                s.setValue(statusValue);

                if (s.containsWall() == true) {
                    t = new WallTile(maze, x, y);
                } else if (s.containsOneWayTile() == true) {
                    t = new OneWayTile(maze, x, y);
                } else {
                    //if the tile is not a WallTile or a OneWayTile, it is an EmptyTile
                    t = new EmptyTile(maze, x, y);
                }
//            setStatus adds a coin or a box if necessary
                t.setStatus(s);

                //adds the tile to the maze
                maze.setTile(t, x, y);

                //prepares x and y values for the next tile
                if (x == 29) {
                    x = 0;
                    y++;
                } else {
                    x++;
                }

            }
            if (scanner.hasNextInt()) {
                //scans pacmans x and y locations
                int pacmanX = scanner.nextInt();
                int pacmanY = scanner.nextInt();


                //adds pacman to maze
                Tile pacmanTile = maze.getTile(pacmanX, pacmanY);
                PacmanEntity pacman = new PacmanEntity(maze);
                pacman.setTile(pacmanTile);
                maze.setPacman(pacman);
            }
            //reads string "stupidghosts"
            scanner.nextLine();
            scanner.nextLine();
            //scans stupidghost locations and adds these to the maze
            while (scanner.hasNextInt()) {
                int stGhostX = scanner.nextInt();
                int stGhostY = scanner.nextInt();
                maze.addStupidGhost(stGhostX, stGhostY);
            }
            //reads string "smartghosts"
            scanner.nextLine();
            scanner.nextLine();
            //scans smartghost locations and adds these to the maze
            while (scanner.hasNextInt()) {
                int smartGhostX = scanner.nextInt();
                int smartGhostY = scanner.nextInt();
                maze.addSmartGhost(smartGhostX, smartGhostY);
            }
            scanner.close();
        } catch (FileNotFoundException e) {
            Popup.ok("Error: File not found: " + filePath);
        }

    }

    /**
     * This method returns the alltime high score list for a mae
     * @param filepath the path of the maze
     * @return the scorelist containing the all time highscores of the maze
     */
    public static ScoreList loadAllTimeScores(String filepath) {
        return readScores(filepath, "all");
    }

    /**
     * This method returns the day high score list for a mae
     * @param filepath the path of the maze
     * @return the scorelist containing today's highscores of the maze
     */
    public static ScoreList loadDayScores(String filepath) {
        return readScores(filepath, "day");
    }

    /**
     * This method returns the day high score list for a mae
     * @param filepath the path of the maze
     * @return the scorelist containing this week's highscores of the maze
     */
    public static ScoreList loadWeekScores(String filepath) {
        return readScores(filepath, "week");
    }

    /**
     * 
     * @param filepath the path of the maze
     * @param listType the desired type of highscore list
     * @return a scorelist containing the requested type of highscorelist for the maze
     */
    public static ScoreList readScores(String filepath, String listType) {
        Scanner scanner;
        File mazefile = new File(filepath);
        ScoreList scorelist = new ScoreList(listType);

        //goes to the area where the desired scores are stored
        try {
            scanner = new Scanner(mazefile);
            while (scanner.nextLine().equals(listType) == false) {
               
            }

            //scans the scores
            for (int i = 1; i < 11; i++) {
                int score = scanner.nextInt();

                String dateString = "" + scanner.nextInt();
                if (dateString.length() == 7){
                    dateString = "0" + dateString;
                }
                //is de datum niet mm-dd-yyyy????? dus maand eerst en dan dag.
                String dayString = "" + dateString.charAt(0) + dateString.charAt(1);
                int day = Integer.parseInt(dayString);
                String monthString = "" + dateString.charAt(2) + dateString.charAt(3);
                int month = Integer.parseInt(monthString);
                String yearString = "" + dateString.charAt(4) + dateString.charAt(5) + dateString.charAt(6) + dateString.charAt(7);
                int year = Integer.parseInt(yearString);

                String name = scanner.nextLine();
                name = name.substring(1); //skip space in front of name
                Calendar date = Calendar.getInstance();
                date.set(year, month, day);
                scorelist.editScore(i, name, score, date);
            }
            scanner.close();

            scorelist.updateList();
            scorelist.sortList();
            return scorelist;

        } catch (FileNotFoundException e) {
            Popup.ok("Error: File not found");
            return new ScoreList(listType);
        }
    }



    /**
     * Creates an empty Maze.
     * @param maze The Maze to load to.
     */
    public static void emptyMaze(Maze maze) { //deze functie kan waarschijnlijk op een veel makkelijkere manier
        int[] statusValues = new int[600];
        int x = 0;
        int y = 0;
        Status s;
        Tile t;

        for (int i = 0; i < 600; i++) {
            statusValues[i] = 0;
        }

        //assigns statusvalues to tiles
        for (int statusValue : statusValues) {
            s = new Status();
            s.setValue(statusValue);

            //er zijn alleen maar lege tiles
            t = new EmptyTile(maze, x, y);

            //adds the tile to the maze
            maze.setTile(t, x, y);

            //prepares x and y values for the next tile
            if (x == 29) {
                x = 0;
                y++;
            } else {
                x++;
            }
        }
    }

       /**
     * Checks if a playable or unplayable file exists with this name
        * @param filepath the name of the mazefile
        * @return boolean wether the file exists or not
     */
    public static boolean doesFileExist(String filepath){

        File fileply = new File(filepath +".ply");
        File fileunp = new File(filepath +".unp");

        if (fileply.exists() || fileunp.exists()){
            return true;
        } else {
            return false;
        }

    }

           /**
     * Checks if a playable or unplayable file exists with this name
     * @param oldpath The path to the existing file
     * @param newpath The path that the file must be copied to
     */
    public static void copyFile(String oldpath, String newpath){
        File oldFile = new File(oldpath);
        File newFile = new File(newpath);

        try {
        Scanner scanner = new Scanner(oldFile);
        PrintWriter pw = new PrintWriter(newFile);

        while (scanner.hasNext()){
            pw.println(scanner.nextLine());
        }

        pw.flush();
        pw.close();
        scanner.close();
        }
        catch (IOException e) {
            Popup.ok("Error: Copying failed");

        }
    }
    /**
     *  Deletes a file from the system
     * @param path to the file that must be deleted
     */
    public static void deleteFile(String path){
        File file = new File(path);
        file.delete();

    }
}
