package biteme;

/**
 * Entity representing the controllable Pacman.
 * @author NFW Thissen
 * @version 0.0.1
 */
public class PacmanEntity extends Entity {

    /** SoundThread object used to play the moving sound. */
    SoundThread st = new SoundThread("sounds\\eating.wav");

    /**
     * Constructor.
     * @param m The GameGUI instance.
     */
    public PacmanEntity(Maze m) {
        super(m);
    }

    /**
     * Moves Pacman in the specified Direction.
     * @param direction The Direction to move in.
     */
    public void move() {
        if (direction != Direction.NONE) {
            this.getTile().moveBox(direction);
            this.getTile().getStatus().clearCoin();
            this.move(direction);

            Sound.playPacmanMoving();
        }
        direction = Direction.NONE;
    }

    /**
     * Calls the draw method of the GameGUI.
     * @param g The Graphics object used for drawing.
     */
    public void draw() {
        String spriteName = "Pacman";
        switch (direction) {
            case UP:
                spriteName += "U";
                break;
            case DOWN:
                spriteName += "D";
                break;
            case LEFT:
                spriteName += "L";
                break;
            case RIGHT:
                spriteName += "R";
                break;
            case NONE:
                spriteName += "U"; //???
                break;
        }

        maze.drawAtXY(spriteName,
                this.getTile().getLocation().x,
                this.getTile().getLocation().y);
    }
}
