package biteme;

import java.io.File;
import java.io.IOException;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;
 

/**
 * Class that plays a sound in a separate thread.
 * @author N.F.W. Thissen
 */
public class SoundThread extends Thread {

    /** Buffer size. */
    private final int EXTERNAL_BUFFER_SIZE = 524288; // 128Kb

    /** Path of the sound file to play. */
    private String waveFile;

    /**
     * Constructor.
     * @param fileName The path of the sound file to play.
     */
    public SoundThread(String fileName) {
        waveFile = fileName;
    }

    /**
     * Runs this thread asynchronously. Do NOT call this method directly, instead call start().
     */
    @Override public void run() {
        File soundFile = new File(waveFile);
        if (!soundFile.exists()) {
            System.err.println("Wave file not found: " + waveFile);
            return;
        }

        AudioInputStream audioInputStream = null;
        try {
            audioInputStream = AudioSystem.getAudioInputStream(soundFile);
        } catch (UnsupportedAudioFileException e1) {
            e1.printStackTrace();
            return;
        } catch (IOException e1) {
            e1.printStackTrace();
            return;
        }

        AudioFormat format = audioInputStream.getFormat();
        SourceDataLine auline = null;
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);

        try {
            auline = (SourceDataLine) AudioSystem.getLine(info);
            auline.open(format);
        } catch (LineUnavailableException e) {
            e.printStackTrace();
            return;
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        auline.start();
        int nBytesRead = 0;
        byte[] abData = new byte[EXTERNAL_BUFFER_SIZE];

        try {
            while (nBytesRead != -1) {
                nBytesRead = audioInputStream.read(abData, 0, abData.length);
                if (nBytesRead >= 0)
                        auline.write(abData, 0, nBytesRead);
            }
        } catch (IOException e) {
            e.printStackTrace();
            return;
        } finally {
            auline.drain();
            auline.close();
        }

    }
}
