package biteme;


/**
 * Tile representing a non-movable wall.
 * @author N.F.W. Thissen, A.M. Zantema
 * @version 0.0.1
 */
public class WallTile extends Tile {

    /**
     * Constructor. GameGUI instance must be provided for drawing purposes.
     * @param m The GameGUI instance.
     */
    public WallTile(Maze m, int x, int y) {
        super(m, x, y);

        //Walltile status always contains a wall:
        this.getStatus().setWall();
    }

    /**
     * Checks if the specified Entity can enter this Tile from the specified Direction.
     * @param d The Direction to check for.
     * @param e The Entity to check.
     * @return False. No Entity can enter a WallTile.
     */
    public boolean canEnter(Direction d, Entity e) {
        return false;
    }

    /**
     * This method checks whether this tile contains a ghost or not.
     * @return false. WallTiles cannot contain ghosts.
     */
    public boolean containsGhost() {
        return false;
    }

    /**
     * Checks whether an entity can leave a tile in a certain direction.
     * @param d The direction in which the entity wants to leave the tile.
     * @param e The entity that wants to leave the tile.
     * @return true. This is chosen arbitrarily, since no entities can enter
     * a wall tile in the first place.
     */
    public boolean canLeave(Direction d, Entity e) {
        return true;
    }

    /**
     * Retrieves the first empty tile to which a box can be moved.
     * @param d
     * @return null. A box cannot move on a WallTile.
     */
    public Tile getNextBoxTile(Direction d) {
        // a box cannot move on a WallTile
        return null;
    }

    /**
     * Calls the draw method in the GameGUI class with appropriate parameters.
     */
    public void draw() {
        maze.drawAtXY("wall", getLocationX(), getLocationY());
    }

}
